#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <iomanip>
#include "wave.h"
#include <SPP/SPP.h>

#ifdef HAVE_DISLIN
#include "dislin.h"
#endif

using namespace std;

int main(int argc, char *argv[])
{
    WaveFile in;
    ofstream archivo;
    string nombre;
    string aux;
    cout << "Enter filename to process (without .wav): " ;
    cin >> nombre;
    aux=nombre+".wav";
    in.OpenRead(aux.c_str());
    if (in.GetError())
    {
        cout << "Error: " << in.GetError() << "." << endl;
        return 1;
    }
    aux=nombre+".log";
    archivo.open(aux.c_str(),fstream::out | fstream::app);
    TUInt32 nsamp;
    TReal64 fs;
    nsamp = in.GetNumSamples();
    fs = in.GetSampleRate();
    PVector64	signal=NULL;
    CreateVector(signal, nsamp);
    TUInt32 i;

    for (i = 0; i < nsamp; i++)
    {
        in.ReadSample(signal[i]);
    }
    in.Close();
    cout << "Sampling Frequency: " << fs << " Hz." << endl;
    PVector64 sch=NULL;
    PMatrix64 RT=NULL;
    TReal64 c50,c80,d50,nf;

    Eval_Rev(signal,fs,sch,nf,RT,c50,c80,d50);
    //cout << "Error: " << LastError() << endl;
    cout << "Noise floor: " << nf << " dB." << endl;
    cout << "Reverberation times: " << endl;
    cout << "Freq.  T20   T30  EDT " << endl;
    cout << RT << endl;
    cout << "C50 = " << c50 << endl;
    cout << "C80 = " << c80 << endl;
    cout << "D50 = " << d50 << endl;

    archivo << "Noise floor: " << nf << " dB." << endl;
    archivo << "Reverberation times: " << RT << endl;
    archivo << "C50 = " << c50 << endl;
    archivo << "C80 = " << c80 << endl;
    archivo << "D50 = " << d50 << endl;

    archivo.close();
#ifdef HAVE_DISLIN
    TUInt32 n,k,q;
    Size(sch,n);
    q=n;
    for (k=0;k<n;k++)
    {
        if isinf(sch[k])
        {
            if (q==n)
                q=k;
            break;
        }
    }
    double t[q];
    for (k=0;k<q;k++)
    {
        t[k]=k/fs;
    }
    //cout << q << endl;
    string tit;
    metafl((char*)"EPS");
    setpag((char*)"PS4L");
    pagmod((char*)"LAND");
    aux=nombre+".eps";
    setfil((char*)aux.c_str());
    disini();
    name((char*)"time [s]",(char*)"X");
    name((char*)"Smoothed decay [dB]",(char*)"Y");
    titlin((char*)"Reverberation time [s]: ",1);
    number((float)RT[0][2],4,1300,130);
    qplot(t,sch,k);

    metafl((char*)"PDF");
    setpag((char*)"PS4L");
    pagmod((char*)"LAND");
    aux=nombre+".pdf";
    setfil((char*)aux.c_str());
    disini();
    name((char*)"time [s]",(char*)"X");
    name((char*)"Smoothed decay [dB]",(char*)"Y");
    titlin((char*)"Reverberation time [s]: ",1);
    number((float)RT[0][2],4,1300,130);
    qplot(t,sch,k);
    //disfin();
#endif

    DestroyVector(signal);
    DestroyVector(sch);
    DestroyMatrix(RT);

    //system("PAUSE");
    return EXIT_SUCCESS;
}
